%% Publishable Experiment to Explain Features of the Four-Bar-Linkage
%  Tim C. Lueth, 2015-02-16, Grindelwald, Switzerland

%%
% The Four-Bar-Linkage is a closed kinematic chain of four sequentially
% linked bars, linked by rotatorial joins. If one of the joints (B) is driven by a
% motor or crank handle, the movement of the bars (b, c, d) is enforced.
% Typically there exist two solutions for the pose of joint D.
% Therefore there exist two solutions for a Four-Bar-Linkage configuration. 
% The equations for the two angles, the general solution for the
% Four-Bar-Linkage and the solving method is described in exp_2015_02_13.
% If there is interest to understand the symbolic equation solution principle
% look there. In this example, we only try to understand the results of the
% Four-Bar-Linkage Kinematic which is very interesting and not easy to
% realize.
function exp_2015_02_16
close all;

%% Draw a specific configuration of 4 bars
% We use the function fourBarLinkage to calculate the 4Bar-Linkage
% configuration or to plot drawings, if we not use result parameters

fourBarLinkage(40,10,40,20,80/180*pi);   % called with length of 4 bars and an angle for the crank handle

%% Calculate the same configuration without a drawing
% The four results of the next function call with the same parameters are:
%%
% * C     Coordinates of the Point C
% * s     true if a solution exist
% * D1    Coordinates of the Point D1 (solution 1)
% * D2    Coordinates of the Point D2 (solution 2)

[C,s,D1,D2]=fourBarLinkage(40,10,40,20,80/180*pi)

%% Calculate and draw the configuration for a complete crank handle turn
% If the 5th parameter is missing or is empty ([]) during a function call,
% the function automatically plots all drawings of a complete turn of the
% crank (b-bar). It takes some seconds to draw about 100 configurations.
%%
% * thin red dashed line    the circle of the crank movement (Point C)
% * thin blue line          the movement of solution one Point D1
% * thin green line         the movement of solution two Point D2
% * thick dashed blue line  1st/last configuration of solution one
% * thick dashed green line 1st/last configuration of solution two
fourBarLinkage(40,10,40,20);

%% Analyze the movement of the coupler
% See also http://www.mekanizmalar.com/fourbar01.html 
% 
% Point C (end of the crank) always describe a circular path with a defined 
% speed give by the crank drive. Point D1/D2 describe also movements one 
% circular pathes, but speed and direction can change during the movement.
% The coupler-bar between C and D describe a combined movement between
% those two pathes. By calling the fourBarLinkage function with an empty
% 5th parameter (no specific angle) and a 6th parameter to specify a point
% on the coupler draws the movement of the coupler point. Defining the
% Point P = C+k*(D-C) leads for k=0 to C and for k=1 to D. Try numbers such
% as 0.5 or 2.
fourBarLinkage(40,10,40,20,[],0.5);

%%
% Try with k=2
fourBarLinkage(40,10,40,20,[],2);

%% Shift the movement orthogonal to the coupler
% By finding a point orthogonal to the direction of the coupler we get an
% additional possibility to modify the shape of the curve. In realty it is not
% necessary to implement it as othogonal bar but a direct line is possible 
% or the implementation as a plate or as a triangle.
%%
% % Try with a point of distance 10 relative to D (k=1)
fourBarLinkage(40,10,40,20,[],1,10);

%%
% Try with a point of distance 10 relative to C (k=0)
fourBarLinkage(40,10,40,20,[],0,10);

%%
% Try with a point of distance 10 between C and D (k=0.5)
fourBarLinkage(40,10,40,20,[],0.5,10);

%% Generate point lists as result
% As shown already above, this function fourBarLinkage is not only for
% drawing linkages but under normal condition just for calculations. By
% using output parameters, the function fourBarLinkage will not generate a
% drawing but returns points or point lists.
fourBarLinkage(40,30,40,30.1,-pi:pi/32:pi);
[C,~,D1,D2,W]=fourBarLinkage(40,30,40,30.1,-pi:pi/4:pi)



