% fourBarLinkage(a,b,c,d,wb,k1,k2) - multi purpose procedure for a 
%	4-Bar-Linkage
%	(by Tim Lueth, VLFL-Lib, 2015-FEB-13 as class: KINEMATICS AND FRAMES)
%	
%	The core of this procedure are the two analytical solutions for the 
%	calculation of the closed 4-Bar-linkage angle. The procedure calculates 
%	for given link length a, b, c, d and an angle for phi-b, the two 
%	solutions for closing the linkage and returns the solution points
%	without an output parameter the configuration is plotted.
%	without input parameter wb, a drawing is created to explain the linkage
%	 (Status of: 2015-02-14)
%	
%	[C,s,D1,D2,wb,wa1,wa2]=fourBarLinkage(a,b,c,d,[wb,k1,k2])
%	=== INPUT PARAMETERS ===
%	a:	base length
%	b:	rotating length
%	c:	linkage bar
%	d:	rotating length
%	wb:	rotation angle at a/b
%	k1:	C+k1*(D1-C)
%	k2:	orthogonal to k1 in mm
%	=== OUTPUT RESULTS ======
%	C:	Point C
%	s:	solution exist or not
%	D1:	Point D1
%	D2:	Point D2
%	wb:	rotation angle at a/b
%	wa1:	rotation angle at a/d (solution 1)
%	wa2:	rotation angle at a/d (solution 2)
%	
%	EXAMPLE: To see the use simply try
%	fourBarLinkage (30,40,30,20);
%	fourBarLinkage (30,40,30,20,[],0.5);
%	fourBarLinkage (30,10,30,20,[],0.5);
%	
function [C,s,D1,D2,wb,wa1,wa2]=fourBarLinkage (a,b,c,d,varargin)

n=50; wb=-1e-5:2*pi/(n-1):2*pi;

        if nargin>=5 && ~isempty(varargin{1}); wb=varargin{1}; end;
k1=inf; if nargin>=6 && ~isempty(varargin{2}); k1=varargin{2}; end;
k2=0;   if nargin>=7 && ~isempty(varargin{3});  k2=varargin{3}; end;
CPLX=[-max(d,b+c) -max(d,b+c); a+b+c -(b+c); a+b+c +(b+c); -max(d,b+c) +max(d,b+c)]; 
         if nargin>=8 && ~isempty(varargin{4});  CPLX=varargin{4}; end;
CPLX=[];
         
if length(wb)>1
    CL=NaN(n,3); D1L=NaN(n,3); D2L=NaN(n,3); WBL=NaN(n,1); PL1=NaN(n,3); PL2=NaN(n,3); 
    S=zeros(n,2);
    k=0; 
%     dw=2*pi/(n-1.002); w=0.1;
%     if isempty(wb); wb=w:dw:2*pi; end;
    n=length(wb);
    wx=wb; wb=[];
    for wi=1:n;
        w=wx(wi);
        k=k+1;
        [CL(k,:),s,D1L(k,:),D2L(k,:),WBL(k)]=fourBarLinkage(a,b,c,d,w); S(k)=s;
        if ~s; k=k-1; end;

        if nargout==0
            fourBarLinkage(a,b,c,d,w); % if ~isempty(CPLX); CPLplot(CPLX,'k.-',2); end;
            
%                 if k>=2 && norm(D2L(k,:)-D1L(k-1,:))<norm(D1L(k,:)-D1L(k-1,:));
%                 [D1L(k,:),D2L(k,:)]=flipvar(D2L(k,:),D1L(k,:)) ;  
%                 end
            if k>0 && s; 
              PL1(k,:)=CL(k,:)+k1*(D1L(k,:)-CL(k,:)); VLplot([CL(k,:);PL1(k,:)],'m-',3); 
              PL2(k,:)=CL(k,:)+k1*(D2L(k,:)-CL(k,:)); VLplot([CL(k,:);PL2(k,:)],'c-',3);  
              u=(D1L(k,:)-CL(k,:)); u=u/norm(u); u=[u(2) -u(1) u(3)]; 
              v=(D2L(k,:)-CL(k,:)); v=v/norm(v); v=[v(2) -v(1) v(3)]; 
              VLplot([PL1(k,:);PL1(k,:)+k2*u],'m-',3); 
              VLplot([PL2(k,:);PL2(k,:)+k2*v],'c-',3); 
              PL1(k,:)=PL1(k,:)+k2*u;
              PL2(k,:)=PL2(k,:)+k2*v; 
            end; 
            VLplot(CL(1:k,:),'r--');VLplot(D1L(1:k,:),'g-',2);VLplot(D2L(1:k,:),'b-');
            VLplot(PL1(1:k,:),'m-'); VLplot(PL2(1:k,:),'c-');
            drawnow;
        end;
    end; 
    CL=CL(1:k,:); D1L=D1L(1:k,:); D2L=D2L(1:k,:);WBL=WBL(1:k); 
    if ~s
        VLplot ([0 0 0; a 0 0; CL(1,:); D1L(1,:);0 0 0],'g*--',3);
%         VLplot ([a 0 0; CL(1,:)],'g*--',3);
%         VLplot ([CL(1,:); D1L(1,:)],'r*--',3);
%         VLplot ([D1L(1,:);0 0 0],'y*--',3); 
        VLplot ([0 0 0; a 0 0; CL(1,:); D2L(1,:);0 0 0],'b*--',3);
    end
%     WBL/pi*180
    title (sprintf('Four-Bar-Linkage (%.1f, %.1f, %1.f, %1.f)\n',a,b,c,d));
    axis auto
    C=CL;D1=D1L;D2=D2L;wb=WBL;
    return; 
end;

%% Calcuate the solutions for wa1/wa2 from wb
wa1=-log((a*b + ((a*b + a^2*exp(wb*1i) + b^2*exp(wb*1i) - c^2*exp(wb*1i) - d^2*exp(wb*1i) + a*b*exp(wb*2*1i) - 2*c*d*exp(wb*1i))*(a*b + a^2*exp(wb*1i) + b^2*exp(wb*1i) - c^2*exp(wb*1i) - d^2*exp(wb*1i) + a*b*exp(wb*2*1i) + 2*c*d*exp(wb*1i)))^(1/2) + a^2*exp(wb*1i) + b^2*exp(wb*1i) - c^2*exp(wb*1i) + d^2*exp(wb*1i) + a*b*exp(wb*2*1i))/(2*d*(b + a*exp(wb*1i))))*1i;
wa2=-log((a*b - ((a*b + a^2*exp(wb*1i) + b^2*exp(wb*1i) - c^2*exp(wb*1i) - d^2*exp(wb*1i) + a*b*exp(wb*2*1i) - 2*c*d*exp(wb*1i))*(a*b + a^2*exp(wb*1i) + b^2*exp(wb*1i) - c^2*exp(wb*1i) - d^2*exp(wb*1i) + a*b*exp(wb*2*1i) + 2*c*d*exp(wb*1i)))^(1/2) + a^2*exp(wb*1i) + b^2*exp(wb*1i) - c^2*exp(wb*1i) + d^2*exp(wb*1i) + a*b*exp(wb*2*1i))/(2*d*(b + a*exp(wb*1i))))*1i;
% if sin(wb); [wa2,wa1]=flipvar(wa1,wa2); end;

%% Calculate the point positions for A B C D1/D2
A=[0; 0; 0];
B=[a; 0; 0]; 
C=[a+b*cos(wb);b*sin(wb); 0];
D1=[d*cos(wa1);d*sin(wa1); 0];
D2=[d*cos(wa2);d*sin(wa2); 0];
if abs(norm(D1-A)-d) < 1e-5; s=true; D1=real(D1); D2=real(D2); wa1=real(wa1); wa2=real(wa2); else s=false; end;
% if D1(2)<0; [D1,D2]=flipvar(D2,D1); end;
if sin(wb)<0; [D1,D2]=flipvar(D2,D1); end;

%% if there is no output argument, plot the result
if nargout==0 % && length(wb)==1
    delete (gca); if ~isempty(CPLX); CPLplot(CPLX,'k.-',2); end;
    axis ([-d a+b -d a+b 0 1]); view (0,90);
    if abs(norm(D1-A)-d)<1e-5; 
         VLELplots(real([A B C D1]'),ELofn(4),'g*-.',4); 
    else VLELplots(real([A B C]'),[1 2;2 3],'r*-.',4); VLELplots([0 0 0;d 0 0;d+c 0 0],[1 2,2 3],'m*-.',5);
    end;
     if abs(norm(D2-A)-d)<1e-5; 
         VLELplots(real([A B C D2]'),ELofn(4),'b*--',3);
     end
    axis equal; grid on; axis ([-d a+b+c -(b+c) b+c 0 1]); view (0,90);
    drawnow;
    show;
end
    
    